/*
 *	Copyright: Torquil Gault (tag@scatha.apana.org.au), 1998+
 *
	Usage Example:

	qdcGL = new QDCGridLayout(10, 50, 5, 5);
	getContentPane().setLayout(qdcGL);
	getContentPane().add(jpanel1);
	qdcGL.setConstraint(jpanel1, new QDCGridLayoutConstraint(0, 0, 11, 5));
	getContentPane().add(jpanel2);
	qdcGL.setConstraint(jpanel2, new QDCGridLayoutConstraint(0, 5, 11, 5));
	getContentPane().add(jpanel3);
	qdcGL.setConstraint(jpanel3, new QDCGridLayoutConstraint(11, 0, 5, 10));
	getContentPane().add(jpanel4);
	qdcGL.setConstraint(jpanel4, new QDCGridLayoutConstraint(16, 0, 34, 2));
	getContentPane().add(jpanel5);
	qdcGL.setConstraint(jpanel5, new QDCGridLayoutConstraint(16, 2, 34, 8));

	Makes a screen like:

		+--------1---------2---------3---------4---------+
		|         |      |             P4                |
		|         |      |                               |
		|   P1    |      |-------------------------------|
		|         |      |                               |
		|         |      |                               |
		----------+  P3  |             P5                |
		|         |      |                               |
		|         |      |                               |
		|   P2    |      |                               |
		|         |      |                               |
		|         |      |                               |
		+--------1---------2---------3---------4---------+

	Things to remember:
		this grid starts at 0,0
 */

package system;

import com.sun.java.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import system.*;

public class QDCGridLayout implements LayoutManager, java.io.Serializable {
    protected int hgap;
    protected int vgap;
    protected int rows;
    protected int cols;
    protected Hashtable hComps;
    protected QDCGridLayoutConstraint gConst;

    public QDCGridLayout() {
	this(1, 0, 0, 0);
	this.hComps = new Hashtable();
	this.gConst = new QDCGridLayoutConstraint();
    }

    public QDCGridLayout(int rows, int cols) {
	this.this(rows, cols, 0, 0);
    }

    public QDCGridLayout(int rows, int cols, int hgap, int vgap) {
	if ((rows == 0) && (cols == 0)) {
	    throw new IllegalArgumentException("rows and cols cannot both be zero");
	}
	this.rows = rows;
	this.cols = cols;
	this.hgap = hgap;
	this.vgap = vgap;
	this.hComps = new Hashtable();
	this.gConst = new QDCGridLayoutConstraint();
    }

    public int getRows() {
	return rows;
    }

    public void setRows(int rows) {
	if ((rows == 0) && (this.cols == 0)) {
	    throw new IllegalArgumentException("rows and cols cannot both be zero");
	}
	this.rows = rows;
    }

    public int getColumns() {
	return cols;
    }

    public void setColumns(int cols) {
	if ((cols == 0) && (this.rows == 0)) {
	    throw new IllegalArgumentException("rows and cols cannot both be zero");
	}
	this.cols = cols;
    }

    public int getHgap() {
	return hgap;
    }
    
    public void setHgap(int hgap) {
	this.hgap = hgap;
    }
    
    public int getVgap() {
	return vgap;
    }
    
    public void setVgap(int vgap) {
	this.vgap = vgap;
    }

    public void setConstraint(Component comp, QDCGridLayoutConstraint con) {
	this.hComps.put(comp, con);
    }

    public void addLayoutComponent(Component comp, QDCGridLayoutConstraint con) {
	this.hComps.put(comp, con);
    }

    public void addLayoutComponent(Component comp, int sX, int sY, int nX, int nY) {
	this.hComps.put(comp, new QDCGridLayoutConstraint(sX, sY, nX, nY));
    }

    public void addLayoutComponent(String str, Component comp) {
    }

    public void addLayoutComponent(Component comp, Object cons) {
	if (cons instanceof QDCGridLayoutConstraint) {
	   this.hComps.put(comp, cons);
	}
    }

    public void removeLayoutComponent(Component comp) {
	this.hComps.remove(comp);
    }

    public Dimension preferredLayoutSize(Container parent) {
	return parent.getSize();
    }

    public Dimension minimumLayoutSize(Container parent) {
	return parent.getSize();
    }

    public void layoutContainer(Container parent) {
      synchronized (parent.getTreeLock()) {
	Insets insets = parent.getInsets();
	int ncomponents = parent.getComponentCount();
	int nrows = rows;
	int ncols = cols;
	int x;
	int y;

	if (ncomponents == 0) {
	    return;
	}
	float w = parent.getWidth() - (insets.left + insets.right);
	float h = parent.getHeight() - (insets.top + insets.bottom);
	w = (float ) (w / cols);
	h = (float ) (h / rows);

	for (int i = 0 ; i < ncomponents ; i++) {
	    Component comp = parent.getComponent(i);
	    gConst = (QDCGridLayoutConstraint )hComps.get(comp);
	    if (gConst != null) {
		x = (int )(gConst.startX * w);
		y = (int )(gConst.startY * h);
		comp.setBounds(x, y, (int )(gConst.noX * w) - vgap, (int )(gConst.noY * h) - hgap);
	    }
	}
      }
    }
    
    public String toString() {
	return getClass().getName() + "[hgap=" + hgap + ",vgap=" + vgap + 
	    			       ",rows=" + rows + ",cols=" + cols + "]";
    }
}
